package builtins_test

import (
	"context"
	"github.com/stretchr/testify/assert"
	"testing"
	"ucl.lmika.dev/ucl"
	"ucl.lmika.dev/ucl/builtins"
)

func TestLists_First(t *testing.T) {
	tests := []struct {
		desc    string
		eval    string
		want    any
		wantErr bool
	}{
		{desc: "firsts 1", eval: `lists:first [1 2 3 4 5] 0`, want: []any{}},
		{desc: "firsts 2", eval: `lists:first [1 2 3 4 5] 3`, want: []any{1, 2, 3}},
		{desc: "firsts 3", eval: `lists:first [1 2 3] 5`, want: []any{1, 2, 3}},
		{desc: "firsts 4", eval: `lists:first [1 2 3] 1`, want: []any{1}},
		{desc: "firsts 5", eval: `lists:first [1 2 3 4 5] -1`, want: []any{1, 2, 3, 4}},
		{desc: "firsts 6", eval: `lists:first [1 2 3 4 5] -3`, want: []any{1, 2}},
		{desc: "firsts 7", eval: `lists:first [1 2 3 4 5] -8`, want: []any{}},
		{desc: "firsts 8", eval: `lists:first (itrs:from [1 2 3 4 5]) 3`, want: []any{1, 2, 3}},
		{desc: "firsts 9", eval: `lists:first (itrs:from [1 2 3]) 5`, want: []any{1, 2, 3}},

		{desc: "err 1", eval: `lists:first (itrs:from [1 2 3]) -3`, wantErr: true},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			inst := ucl.New(
				ucl.WithModule(builtins.Itrs()),
				ucl.WithModule(builtins.Lists()),
			)
			res, err := inst.EvalString(context.Background(), tt.eval)
			if tt.wantErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.want, res)
			}
		})
	}
}

func TestLists_Uniq(t *testing.T) {
	tests := []struct {
		desc    string
		eval    string
		want    any
		wantErr bool
	}{
		{desc: "uniq 1", eval: `lists:uniq [a a a a b b b c c c]`, want: []any{"a", "b", "c"}},
		{desc: "uniq 2", eval: `lists:uniq [1 2 1 3 2 4 2 5 3]`, want: []any{1, 2, 3, 4, 5}},
		{desc: "uniq 3", eval: `lists:uniq [1 a 2 b 3 b 2 a 1]5`, want: []any{1, "a", 2, "b", 3}},

		{desc: "uniq err 1", eval: `lists:uniq [[1 2 3] [a:2] ()]`, wantErr: true},
	}

	for _, tt := range tests {
		t.Run(tt.desc, func(t *testing.T) {
			inst := ucl.New(
				ucl.WithModule(builtins.Itrs()),
				ucl.WithModule(builtins.Lists()),
			)
			res, err := inst.EvalString(context.Background(), tt.eval)
			if tt.wantErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.want, res)
			}
		})
	}
}

func uclListOf(args ...any) *ucl.ListObject {
	newList := ucl.NewListObject()
	for _, arg := range args {
		switch t := arg.(type) {
		case int:
			newList.Append(ucl.IntObject(t))
		default:
			panic("unhandled type")
		}
	}
	return newList
}
